<?php

namespace Filoblu\Refilo\Model\Indexer\Core;

use Filoblu\Refilo\Model\Adapter\Mongo;
use Filoblu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterface;
use Filoblu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterfaceFactory;
use Magento\Framework\Mview\ActionInterface;

abstract class AbstractMongoIndexer implements \Magento\Framework\Indexer\ActionInterface, ActionInterface
{
    /**
     * @var SqlToMongoAdapterInterface
     */
    protected $sqlToMongoAdapter;
    /**
     * @var Mongo
     */
    protected $mongo;
    /**
     * @var QueryProviderInterfaceFactory
     */
    protected $queryProviderFactory;

    /**
     * AbstractMongoIndexer constructor.
     * @param SqlToMongoAdapterInterface $sqlToMongoAdapter
     * @param QueryProviderInterfaceFactory $queryProviderFactory
     * @param Mongo $mongo
     */
    public function __construct(
        SqlToMongoAdapterInterface $sqlToMongoAdapter,
        QueryProviderInterfaceFactory $queryProviderFactory,
        Mongo $mongo
    ) {
        $this->sqlToMongoAdapter = $sqlToMongoAdapter;
        $this->mongo = $mongo;
        $this->queryProviderFactory = $queryProviderFactory;
    }

    /**
     * @inheritDoc
     */
    public function executeFull()
    {
        $this->performAction();
    }

    protected function performAction($ids = null)
    {
        /** @var QueryProviderInterface $provider */
        $provider = $this->queryProviderFactory->create();
        $provider->filter($ids);

        $result = $this->sqlToMongoAdapter->adaptFromSql($provider);
        $this->mongo->update($this->getMongoKey(), $result, $this->getTargetMongoCollection());
    }

    abstract public function getMongoKey(): string;

    abstract public function getTargetMongoCollection(): string;

    abstract public function setInsertMode(): bool;

    /**
     * @inheritDoc
     */
    public function executeRow($id)
    {
        $this->executeList([$id]);
    }

    /**
     * @inheritDoc
     */
    public function executeList(array $ids)
    {
        $this->performAction($ids);
    }

    /**
     * @inheritDoc
     */
    public function execute($ids)
    {
        $this->executeList($ids);
    }
}
