<?php
/**
 * Created by PhpStorm.
 * User: cesposito
 * Date: 1/24/20
 * Time: 11:51 AM
 */

namespace Filoblu\Refilo\Model\Indexer\Processor\Blocks;

use Magento\CatalogWidget\Model\RuleFactory;
use Magento\Framework\Data\Wysiwyg\Normalizer as FilterNormalized;
use Magento\Framework\Filter\Template as FilterTemplate;
use Magento\Framework\Serialize\Serializer\Json as SerializerJson;
use Filoblu\Refilo\Model\Indexer\Processor\Blocks\Rendered\Conditions;

class Renderer
{

    /**
     * @var SerializerJson
     */
    private $serializerJson;
    /**
     * @var RuleFactory
     */
    private $rule;
    /**
     * @var FilterTemplate
     */
    private $filterTemplate;
    /**
     * @var FilterNormalized
     */
    private $filterNormalizer;

    /**
     * @var array
     */
    private $renderer_classes = [
//        "Magento\VersionsCms\Block\Widget\Node" => ["name" => "CMS Hierarchy Node Link","source"=>"pages","prk"=>"page_id"],
        "Magento\Cms\Block\Widget\Page\Link" => ["name" => "CMS Page Link","source"=>"pages","prk"=>"page_id"],
        "Magento\Cms\Block\Widget\Block" => ["name" => "CMS Static Block","source"=>"blocks","prk"=>"block_id"],
        "Magento\Catalog\Block\Category\Widget\Link" => ["name" => "Catalog Category Link","source"=>"categories","prk"=>"id_path"],
//        "Magento\CatalogEvent\Block\Widget\Lister" => ["name" => "Catalog Events Carousel"],
//        "Magento\Catalog\Block\Product\Widget\NewWidget" => ["name" => "Catalog New Products List","source"=>"pages","prk"=>"page_id"],
//        "Magento\Catalog\Block\Product\Widget\Link" => ["name" => "Catalog Product Link","source"=>"pages","prk"=>"page_id"],
        "Magento\CatalogWidget\Block\Product\ProductsList" => ["name" => "Catalog Products List","source"=>"products"],
//        "Blackbird\ContentManager\Block\Content\Widget\Link" => ["name" => "Content Manager Content Link","source"=>"block"],
        "Blackbird\ContentManager\Block\Content\Widget\ContentList" => ["name" => "Content Manager Content List","source"=>"block"],
//        "Blackbird\ContentManager\Block\Content\Widget\View" => ["name" => "Content Manager Content View","source"=>"block"],
//        "FiloBlu\Design\Block\Widget\CustomWidget" => ["name" => "Custom Widget","source"=>"pages","prk"=>"page_id"],
//        "Magento\Banner\Block\Widget\Banner" => ["name" => "Dynamic Blocks Rotator","source"=>"pages","prk"=>"page_id"],
//        "FiloBlu\Countdown\Block\Widget\Countdown" => ["name" => "FB Countdown","source"=>"pages","prk"=>"page_id"],
//        "FiloBlu\ScratchAndWin\Block\Widget\ScratchAndWin" => ["name" => "FB ScratchAndWin","source"=>"pages","prk"=>"page_id"],
//        "FiloBlu\Popup\Block\Widget\Popup" => ["name" => "FiloBlu Popup","source"=>"pages","prk"=>"page_id"],
//        "Magento\GiftRegistry\Block\Search\Widget\Form" => ["name" => "Gift Registry Search","source"=>"pages","prk"=>"page_id"],
//        "Magento\AdvancedCheckout\Block\Widget\Sku" => ["name" => "Order by SKU","source"=>"pages","prk"=>"page_id"],
//        "Magento\Sales\Block\Widget\Guest\Form" => ["name" => "Orders and Returns","source"=>"pages","prk"=>"page_id"],
//        "Magento\Catalog\Block\Widget\RecentlyCompared" => ["name" => "Recently Compared Products","source"=>"pages","prk"=>"page_id"],
//        "Magento\Catalog\Block\Widget\RecentlyViewed" => ["name" => "Recently Viewed Products","source"=>"pages","prk"=>"page_id"],
//        "Magento\MultipleWishlist\Block\Widget\Search" => ["name" => "Wish List Search","source"=>"pages","prk"=>"page_id"]
    ];
    /**
     * @var Conditions
     */
    private $conditionBuilder;

    /**
     * Catalog constructor.
     * @param FilterTemplate $filterTemplate
     * @param SerializerJson $serializerJson
     * @param RuleFactory $ruleFactory
     * @param FilterNormalized $filterNormalizer
     * @param Conditions $conditions
     */
    public function __construct(
        FilterTemplate $filterTemplate,
        SerializerJson $serializerJson,
        RuleFactory $ruleFactory,
        FilterNormalized $filterNormalizer,
        Conditions $conditions
    ) {
        $this->serializerJson = $serializerJson;
        $this->rule = $ruleFactory->create();
        $this->filterTemplate = $filterTemplate;
        $this->filterNormalizer = $filterNormalizer;
        $this->conditionBuilder = $conditions;
    }

    public function renderBlock($block_params)
    {
        $metacontent = new \stdClass();
        $metacontent->query = new \stdClass();

        if (isset($block_params['template'])) {
            unset($block_params['template']);
        }

        if (!isset($this->renderer_classes[$block_params["class"]])) {
            return "";
        }

        $class = str_replace("\\", "_", $block_params["class"]);

        $metacontent->class = $class;
        $metacontent->source = null;
        $metacontent->metadata = (object)$block_params;

        return $metacontent;
    }

    public function renderWidget($block_params)
    {
        if (isset($block_params['template'])) {
            unset($block_params['template']);
        }

        if (!isset($this->renderer_classes[$block_params["type"]])) {
            return "";
        }
        $class = str_replace("\\", "_", $block_params["type"]);
        $source = $this->renderer_classes[$block_params["type"]]["source"];

        $metacontent = new \stdClass();
        $metacontent->query = new \stdClass();

        if (isset($this->renderer_classes[$block_params["type"]]["prk"]) && isset($block_params[$this->renderer_classes[$block_params["type"]]["prk"]])) {
            $id = $block_params[$this->renderer_classes[$block_params["type"]]["prk"]];
            $metacontent->prk = $id;
            unset($block_params[$this->renderer_classes[$block_params["type"]]["prk"]]);

        }

        /** @var TYPE_NAME $params */
        if (isset($block_params["conditions_encoded"])) {
            $conditions = $this->conditionBuilder->process($block_params["conditions_encoded"]);
            unset($block_params["conditions_encoded"]);
            $metacontent->query = $conditions;
        }

        unset($block_params["type"]);

        $metacontent->class = $class;
        $metacontent->source = $source;
        $metacontent->metadata = (object)$block_params;

        return $metacontent;

    }
}
