<?php

namespace Filoblu\Refilo\Model\Indexer\Processor;

use Blackbird\ContentManager\Model\ResourceModel\Content\CollectionFactory as BlackbirdItems;
use Magento\Framework\App\ResourceConnection;

/**
 * @property  searchCriteria
 */
class ContentType
{
    private $blackbirdItems;

    /**
     * @var int
     */
    private $totalPages = 0;

    /**
     * @var ResourceConnection
     */
    private $connection;

    /**
     * @var array
     */
    private $private_keys = ["identifier"=>"identifier","url_key"=>"url_key","entity_id"=>"entity_id","title"=>"title","status"=>"status","ct_id"=>"ct_id"];

    /**
     * Catalog constructor.
     * @param ResourceConnection $connection
     * @param BlackbirdItems $blackbirdItems
     */
    public function __construct(
        ResourceConnection $connection,
        BlackbirdItems $blackbirdItems
    ) {
        $this->connection = $connection->getConnection();
        $this->blackbirdItems = $blackbirdItems;
    }

    public function getItems($store, $batch_size, $batch_sequence, $ids = false)
    {
        $contentTypeCollection = $this->prepareItemsCollection($store, $batch_size, $batch_sequence, $ids);
        return ($contentTypeCollection) ? $this->processItemsCollection($contentTypeCollection, $store) : false;
    }

    /**
     * @param $store
     * @param $batch_size
     * @param $batch_sequence
     * @param bool $ids
     * @return \Blackbird\ContentManager\Model\ResourceModel\Content\Collection|bool
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function prepareItemsCollection($store, $batch_size, $batch_sequence, $ids = false)
    {
        if (($this->totalPages < $batch_sequence) && $this->totalPages != 0) {
            return false;
        }

        $collection = $this->blackbirdItems->create();

        $collection->addStoreFilter($store);

        $collection->addAttributeToSelect('*')->getSelect()->joinLeft(
            ['contenttype' => "blackbird_contenttype"],
            'contenttype.ct_id = e.ct_id',
            'identifier'
        );

        if (is_array($ids)) {
            $collection->addFilter('entity_id', $ids, 'in');
        }

        $collection->setPageSize($batch_size);
        $collection->setCurPage($batch_sequence);
        printf("Prepare Collection\n\r");

        if (!$this->totalPages) {
            $count = $collection->count();
            $this->totalPages = ceil($count / $batch_size);
        }
        printf("Page %d of %d \n\r", $batch_sequence, $this->totalPages);
        return $collection;
    }

    /**
     * @param $_contentType_collection
     * @param $store
     * @return array
     */
    protected function processItemsCollection($_contentType_collection, $store)
    {
        $items = [];

        foreach ($_contentType_collection->getItems() as $item) {
            $block = new \stdClass();

            $block->_id = "content_" . $item->getId() . "_" . $store->getData('code');
            $block->metacode = "block";
            $block->store = $store->getData('code');
            $block->metadata = new \stdClass();

            $block->content_id = (int) $item->getId();
            $block->url_key = $item->getUrlKey();
            $block->idx = $item->getUrlKey();
            $block->title = $item->getTitle();
            $block->type = $item->getIdentifier();
            $block->status = $item->getStatus();
            $block->metadata = array_diff_key($item->getData(), $this->private_keys);

            $block->content = "";

            $items[] = $block;
            unset($block);
        }

        return $items;
    }

    /**
     * @return int
     */
    public function getTotalPages()
    {
        return $this->totalPages;
    }

    /**
     * @param int $total_pages
     */
    public function setTotalPages($totalPages)
    {
        $this->totalPages = $totalPages;
    }
}
